/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/gdif/om_util.c#1 $
*/

#include <stdio.h>

#include <avs/dll_in.h>
#include <avs/util.h>
#include <avs/err.h>
#include <avs/om.h>
#include <avs/om_type.h>
#include <avs/fld.h>
#include <avs/gd.h>

OMobj_id GDcreate_instance(elem_id, templ_name, inst_name)
OMobj_id elem_id;
char *templ_name;
char *inst_name;
{
   int ind;
   OMobj_id parent_id, templ_id, inst_id;
   OMobj_name elem_name;

   /* find template the user asked for */
   templ_id = OMfind_str_subobj(OMtempl_obj, templ_name, OM_OBJ_RD);
   if (OMis_null_obj(templ_id))
      return(OMnull_obj);

   /* Create the instance using the elem_id the user specified
      as the parent if it isn't null else use Instances as the parent.
   */
   if (OMis_null_obj(elem_id))
      parent_id = OMinst_obj;
   else parent_id = elem_id;

   /* Make an instance of the template element */
   inst_id = OMcopy_obj(templ_id, OM_NULL_NAME,  parent_id, OMlocal_proc_id,
			OM_COPY_CLOSE);
   if (OMis_null_obj(inst_id))
      return(OMnull_obj);

   /* Give the instance the name the user specified. */
   OMset_obj_name(inst_id, OMstr_to_name(inst_name));
   return(inst_id);
}

void GDset_local(id, local_ptr)
OMobj_id id;
char *local_ptr;
{
   OMobj_id ptr_id;

   /* find pointer to local data structure */
   ptr_id = OMfind_subobj(id, OMstr_to_name("local_ptr"), OM_OBJ_RW);
   if (!OMis_null_obj(ptr_id)) {
      /* set pointer to structure in data layer */
      OMset_ptr_val(ptr_id, local_ptr, 0);
   }
}

/* This routine will break the connection to the local pointer.
   This is used when we get the deinstance function to invalidate
   the local pointer so the next time we get instanced we will
   behave properly.
*/
void GDclear_local(id)
OMobj_id id;
{
   OMobj_id ptr_id;

   /* find pointer to local data structure */
   ptr_id = OMfind_subobj(id, OMstr_to_name("local_ptr"), OM_OBJ_RW);
   if (!OMis_null_obj(ptr_id)) {
      /* set pointer to null obj which will break the connection */
      OMset_obj_val(ptr_id, OMnull_obj, 0);
   }
}

char *GDget_local(id, create_func)
OMobj_id id;
OMpfi create_func;
{
   OMobj_id level_id, ptr_id;
   char *local_ptr;
   int stat;

   ptr_id = OMfind_subobj(id, OMstr_to_name("local_ptr"), OM_OBJ_RW);
   if (OMis_null_obj(ptr_id)) {
      ERRerror("GDget_local", 1, ERR_ORIG, "Can't find local pointer\n");
      return(NULL);
   }

   if ((stat = OMget_ptr_val(ptr_id, (void *)&local_ptr, 0)) == 1)
      /* return pointer to local structure */
      return(local_ptr);
   else {
      /*
       * This is likely because we were given a remote object... we can't
       * deal with that case at all...
       */
      if (stat == -1) return(NULL);
      /* local pointer not valid - this means we need to
	 call the element's create function directly to setup
	 the local pointer, add notifies for the element, etc.
      */
      (*create_func)(id);
      /* now we should be able to get the local pointer */
      if (OMget_ptr_val(ptr_id, (void *)&local_ptr, 0) == 1)
	 return(local_ptr);
      else {
         ERRerror("GDget_local", 1, ERR_ORIG, "Can't create local pointer\n");
	 return(NULL);
      }
   }
}

void GDset_refer_db(id, name, refer_id)
OMobj_id id;
char *name;
OMobj_id refer_id;
{
   OMobj_id elem_id;

   /* attach refer_id to id in database */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (!OMis_null_obj(elem_id))
      OMset_obj_ref(elem_id, refer_id, 0);
}

int GDget_refer_db(id, name, refer_id)
OMobj_id id;
char *name;
OMobj_id *refer_id;
{
   OMobj_id elem_id;

   /* attach refer_id to id in database */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return(0);
   else return(OMget_obj_val_user(elem_id, refer_id));
}

int GDget_seqref_db(id, name, mode, seq, refer_id)
OMobj_id id;
char *name;
int mode;
int *seq;
OMobj_id *refer_id;
{
   OMobj_id elem_id;

   /* attach refer_id to id in database */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return(0);
   else {
      *seq = OMget_obj_seq(elem_id, OMnull_obj, mode);
      return(OMget_obj_val(elem_id, refer_id));
   }
}

int GDget_obj_seq(id, name, seq, mode)
OMobj_id id;
char *name;
int *seq;
int mode;
{
   OMobj_id elem_id;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);
   *seq = OMget_obj_seq(elem_id, OMnull_obj, mode);
   return(1);
}

void GDset_int_val(id, name, value)
OMobj_id id;
char *name;
int value;
{
   OMobj_id elem_id;

   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id)) {
      return;
   }
   OMset_int_val(elem_id, value);
}

int GDget_int_val(id, name, value)
OMobj_id id;
char *name;
int *value;
{
   OMobj_id elem_id;

   *value = 0;
   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   if ((OMget_int_val(elem_id, value) == OM_STAT_SUCCESS)) {
      return(1);
   }
   else return(0);
}

/* This routine sets the value of the requested sub-element
   given the name of the element, and updates the sequence number.
*/
void GDset_newint_val(id, name, seq, value)
OMobj_id id;
char *name;
int *seq;
int value;
{
   OMobj_id elem_id;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return;
   OMset_int_val(elem_id, value);

   /* return updated sequence number */
   *seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
}

/* This routine returns the value of the requested sub-element
   given the name of the element, the current sequence number
   and a pointer to the return data. Return status is 1
   if the data has changed. */
int GDget_newint_val(id, name, seq, value)
OMobj_id id;
char *name;
int *seq;
int *value;
{
   OMobj_id elem_id;
   int elem_seq, ret_val = 0;

   *value = 0;
   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is different set return value */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq != *seq) {
      *seq = elem_seq;		/* update element sequence number */
      ret_val = 1;
   }
   if ((OMget_int_val(elem_id, value) == OM_STAT_SUCCESS)) {
      return(ret_val);
   }
   else return(0);
}

/* This routine returns the value of the requested sub-element
   given the name of the element.  Return status is 1 if the
   passed sequence number is less than the sequence number
   of the requested element.
*/
int GDget_updint_val(id, name, seq, value)
OMobj_id id;
char *name;
int seq;
int *value;
{
   OMobj_id elem_id;
   int elem_seq, ret_val = 0;

   *value = 0;
   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is different set return value */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq > seq)
      ret_val = 1;

   if ((OMget_int_val(elem_id, value) == OM_STAT_SUCCESS)) {
      return(ret_val);
   }
   else return(0);
}

/* This routine returns the value of the requested sub-element
   given the name of the element.  Return status is 1 if the 
   passed sequence number is less than the sequence number
   of the requested element.
*/
int GDget_updstr_val(id, name, seq, value)
OMobj_id id;
char *name;
int seq;
char **value;
{
   OMobj_id elem_id;
   int elem_seq, ret_val = 0;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is different set return value */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq > seq)
      ret_val = 1;

   if ((OMget_str_val(elem_id, value, 0) == OM_STAT_SUCCESS)) {
      return(ret_val);
   }
   else return(0);
}

#ifdef USE_PROTOS
void GDset_float_val(OMobj_id id, char *name, float value)
#else
void GDset_float_val(id, name, value)
OMobj_id id;
char *name;
float value;
#endif
{
   OMobj_id elem_id;

   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return;
   OMset_real_val(elem_id, value);
}

int GDget_float_val(id, name, value)
OMobj_id id;
char *name;
float *value;
{
   OMobj_id elem_id;
   int elem_seq;
   double tmp_val;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);

   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   if ((OMget_real_val(elem_id, &tmp_val) == OM_STAT_SUCCESS)) {
      *value = tmp_val;
      return(1);
   }
   else return(0);
}

/* This routine sets the value of the requested sub-element
   given the name of the element, and updates the sequence number.
*/
#ifdef USE_PROTOS
void GDset_newfloat_val(OMobj_id id, char *name, int *seq, float value)
#else
void GDset_newfloat_val(id, name, seq, value)
OMobj_id id;
char *name;
int *seq;
float value;
#endif
{
   OMobj_id elem_id;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return;
   OMset_real_val(elem_id, value);

   /* return updated sequence number */
   *seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
}

/* This routine returns the value of the requested sub-element
   given the name of the element, the current sequence number
   and a pointer to the return data. Return status is 1
   if the data has changed. */
int GDget_newfloat_val(id, name, seq, value)
OMobj_id id;
char *name;
int *seq;
float *value;
{
   OMobj_id elem_id;
   int elem_seq, ret_val = 0;
   double tmp_val;

   *value = 0.0;
   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is different set return value */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq != *seq) {
      *seq = elem_seq;		/* update element sequence number */
      ret_val = 1;
   }
   if ((OMget_real_val(elem_id, &tmp_val) == OM_STAT_SUCCESS)) {
      *value = tmp_val;
      return(ret_val);
   }
   else return(0);
}

/* This routine returns the value of the requested sub-element
   given the name of the element.  Return status is 1 if the
   passed sequence number is less than the sequence number
   of the requested element.
*/
int GDget_updfloat_val(id, name, seq, value)
OMobj_id id;
char *name;
int seq;
float *value;
{
   OMobj_id elem_id;
   int elem_seq, ret_val = 0;
   double tmp_val;

   *value = 0.0;
   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is different set return value */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq > seq)
      ret_val = 1;

   if ((OMget_real_val(elem_id, &tmp_val) == OM_STAT_SUCCESS)) {
      *value = tmp_val;
      return(ret_val);
   }
   else return(0);
}

void GDset_int_array(id, name, size, values)
OMobj_id id;
char *name;
int size;		/* number of elements - NOT size in bytes */
int *values;
{
   OMobj_id elem_id;

   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return;
   OMset_array(elem_id, OM_TYPE_INT, (char *)values, size,
      OM_SET_ARRAY_COPY);
}

/* This routine returns the value of the requested sub-element
   given the name of the element, the current sequence number
   and a pointer to the return data. Return status is 1
   if the data has changed. */
int GDget_newint_array(id, name, seq, size, values)
OMobj_id id;
char *name;
int *seq;
int *size;
int **values;
{
   OMobj_id elem_id;
   int type = OM_TYPE_INT, elem_seq, ret_val = 0;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(ret_val);

   /* if sequence number is the same return 1 */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq != *seq) {
      *seq = elem_seq;		/* update element sequence number */
      ret_val = 1;
   }
   *size = 0;
   *values = (int *)NULL;
   if ((OMget_array_sz(elem_id, &type, (char **)values, size,
	OM_GET_ARRAY_RD_COPY) == OM_STAT_SUCCESS)) {
      /* If we are returning a zero value but we also got the array,
         we need to free it because the caller will not.
       */
      if (ret_val != 1 && *values != NULL) ARRfree(*values);
      return(ret_val);
   }
   else return(0);
}

/* This routine returns the value of the requested sub-element
   given the name of the element, the current sequence number
   and a pointer to the return data. Return status is 1
   if the data has changed. */
int GDget_updint_array(id, name, seq, size, values)
OMobj_id id;
char *name;
int seq;
int *size;
int **values;
{
   OMobj_id elem_id;
   int type = OM_TYPE_INT, elem_seq, ret_val = 0;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(ret_val);

   /* if sequence number is the same return 1 */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq > seq)
      ret_val = 1;

   *size = 0;
   *values = (int *)NULL;
   if ((OMget_array_sz(elem_id, &type, (char **)values, size,
	OM_GET_ARRAY_RD_COPY) == OM_STAT_SUCCESS)) {
      /* If we are returning a zero value but we also got the array,
         we need to free it because the caller will not.
       */
      if (ret_val != 1 && *values != NULL) ARRfree(*values);
      return(ret_val);
   }
   else return(0);
}

int GDget_int_array(id, name, size, values)
OMobj_id id;
char *name;
int *size;
int **values;
{
   OMobj_id elem_id;
   int type = OM_TYPE_INT, elem_seq;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   *size = 0;
   *values = (int *)NULL;
   /* Ask for the array as RD_COPY so if the user changes the
      returned array, it won't actually changes the values in
      the OM.
   */
   if ((OMget_array_sz(elem_id, &type, (char **)values, size,
	OM_GET_ARRAY_RD_COPY) == OM_STAT_SUCCESS))
      return(1);
   else return(0);
}

void GDset_float_array(id, name, size, values)
OMobj_id id;
char *name;
int size;		/* number of elements - NOT size in bytes */
float *values;
{
   OMobj_id elem_id;

   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   if (OMis_null_obj(elem_id))
      return;
   OMset_array(elem_id, OM_TYPE_FLOAT, (char *)values, size,
      OM_SET_ARRAY_COPY);
}

/* This routine returns the value of the requested sub-element
   given the name of the element, the current sequence number
   and a pointer to the return data. Return status is 1
   if the data has changed. */
int GDget_newfloat_array(id, name, seq, size, values)
OMobj_id id;
char *name;
int *seq;
int *size;
float **values;
{
   OMobj_id elem_id;
   int type = OM_TYPE_FLOAT, elem_seq, ret_val = 0;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is the same just return */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq != *seq) {
      *seq = elem_seq;		/* update element sequence number */
      ret_val = 1;
   }
   *size = 0;
   *values = (float *)NULL;
   if ((OMget_array_sz(elem_id, &type, (char **)values, size,
	OM_GET_ARRAY_RD_COPY) == OM_STAT_SUCCESS)) {
      /* If we are returning a zero value but we also got the array,
         we need to free it because the caller will not.
       */
      if (ret_val != 1 && *values != NULL) ARRfree(*values);
      return(ret_val);
   }
   else return(0);
}

/* This routine returns the values of the requested sub-element
   given the name of the element.  Return status is 1 if the 
   passed sequence number is less than the sequence number 
   of the requested element.
*/
int GDget_updfloat_array(id, name, seq, size, values)
OMobj_id id;
char *name;
int seq;
int *size;
float **values;
{
   OMobj_id elem_id;
   int type = OM_TYPE_FLOAT, elem_seq, ret_val = 0;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* See if this element has actually changed. */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq > seq)
      ret_val = 1;

   *size = 0;
   *values = (float *)NULL;
   if ((OMget_array_sz(elem_id, &type, (char **)values, size,
	OM_GET_ARRAY_RD_COPY) == OM_STAT_SUCCESS)) {
      /* If we are returning a zero value but we also got the array,
         we need to free it because the caller will not.
       */
      if (ret_val != 1 && *values != NULL) ARRfree(*values);
      return(ret_val);
   }
   else return(0);
}

int GDget_float_array(id, name, size, values)
OMobj_id id;
char *name;
int *size;
float **values;
{
   OMobj_id elem_id;
   int type = OM_TYPE_FLOAT;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);

   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   *size = 0;
   *values = (float *)NULL;
   if ((OMget_array_sz(elem_id, &type, (char **)values, size,
	OM_GET_ARRAY_RD_COPY) == OM_STAT_SUCCESS))
      return(1);
   else return(0);
}

int GDget_str_val(id, name, value)
OMobj_id id;
char *name;
char **value;
{
   OMobj_id elem_id;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   if ((OMget_str_val(elem_id, value, 0) == OM_STAT_SUCCESS)) {
      return(1);
   }
   else return(0);
}
void GDset_newstr_val(id, name, seq, value)
OMobj_id id;
char *name;
int *seq;
char *value;
{
   OMobj_id elem_id;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RW);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return;

   OMset_str_val(elem_id, value);

   /* return updated sequence number */
   *seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
}

/* This routine returns the value of the requested sub-element
   given the name of the element, the current sequence number
   and a pointer to the return data. Return status is 1
   if the data has changed. */
int GDget_newstr_val(id, name, seq, value)
OMobj_id id;
char *name;
int *seq;
char **value;
{
   OMobj_id elem_id;
   int elem_seq, ret_val = 0;

   /* lookup element id of requested sub-element */
   elem_id = OMfind_subobj(id, OMstr_to_name(name), OM_OBJ_RD);
   /* return if null element */
   if (OMis_null_obj(elem_id))
      return(0);

   /* if sequence number is different set return value */
   elem_seq = OMget_obj_seq(elem_id, OMnull_obj, OM_SEQ_VAL);
   if (elem_seq != *seq) {
      *seq = elem_seq;		/* update element sequence number */
      ret_val = 1;
   }
   if ((OMget_str_val(elem_id, value, 0) == OM_STAT_SUCCESS)) {
      return(ret_val);
   }
   else return(0);
}

